
#ifndef DEF_H
#define DEF_H


/* ************************************************************
 rappresentazione di una scacchiera 0x88 in esadecimale,
 il primo numero rappresenta la traversa e il secondo la
 colonna. Ha 120 elementi !
   
   -------------------------
8 | 70 71 72 73 74 75 76 77 | TRA 7
7 | 60 61 62 63 64 65 66 67 | 68 69 6a 6b 6c 6d 6e 6f    TRA 6
6 | 50 51 52 53 54 55 56 57 | 58 59 5a 5b 5c 5d 5e 5f    TRA 5
5 | 40 41 42 43 44 45 46 47 | 48 49 4a 4b 4c 4d 4e 4f    TRA 4
4 | 30 31 32 33 34 35 36 37 | 38 39 3a 3b 3c 3d 3e 3f    TRA 3
3 | 20 21 22 23 24 25 26 27 | 28 29 2a 2b 2c 2d 2e 2f    TRA 2
2 | 10 11 12 13 14 15 16 17 | 18 19 1a 1b 1c 1d 1e 1f    TRA 1
1 |  0  1  2  3  4  5  6  7 |  8  9  a  b  c  d  e  f    TRA 0
   -------------------------
     a  b  c  d  e  f  g  h
COL  0  1  2  3  4  5  6  7
**************************************************************   */

#define VERSIONE            "0.172 JA"

#define TABLESIZE           4194304  // 64 Mb dimensione hash tables 

#define MOSSE_STANDARD	    40

#define FATTORE_DI_RICERCA	2        

///////////////////////////////////////////////////////////////////

#define SUPERSPEED           2

#define NO_FLAG           0x00

// flag delle hash tables
#define UPPER                1
#define LOWER                2
#define EXACT                3

// valore in
#define VAL_PE             100
#define VAL_CA             305
#define VAL_AL             305
#define VAL_TO             500
#define VAL_DO             900
#define VAL_RE               0 
#define INFINITO         10000

#define EXIT_SEARCH      32000

#define MAX_PLY           100
#define MAX_HPLY          1000

#define FINALE            ( 5 * VAL_AL )

#define U64               unsigned long long int
#define U8                unsigned char

#define OFF_BOARD         0x8       // valore arbitrario di una casa fuori scacchiera

#define ARROCCO_BC        0x01      // arrocco bianco corto 
#define ARROCCO_BL		  0x02      // arrocco bianco lungo
#define ARROCCO_NC		  0x04      // arrocco nero corto
#define ARROCCO_NL		  0x08      // arrocco nero lungo
#define ARROCCO           0x0F      // tutti gli arrocchi 

#define PROMO_C           0x10      // promozione a cavallo
#define PROMO_A           0x20      // promozione a alfiere
#define PROMO_T           0x40      // promozione a torre
#define PROMO_D           0x80      // promozione a donna
#define PROMOZIONE        0xF0      // tutte le promozioni

#define SPINTA_DUE       0x100
#define CATTURA_EP       0x200
#define CATTURA          0x400

#define FLAG_PEDONE       0x01
#define FLAG_CAVALLO      0x02
#define FLAG_RE           0x04
#define FLAG_ALFIERE      0x08
#define FLAG_TORRE        0x10
#define FLAG_DONNA        0x20
#define FLAG_BIANCO       0x40
#define FLAG_NERO         0x80

#define FLAG_PEZZO        0x3F
#define FLAG_COLORE       0xC0


typedef enum { A1 = 0x00,B1,C1,D1,E1,F1,G1,H1 } COORDINATE_1;
typedef enum { A2 = 0x10,B2,C2,D2,E2,F2,G2,H2 } COORDINATE_2;
typedef enum { A3 = 0x20,B3,C3,D3,E3,F3,G3,H3 } COORDINATE_3;
typedef enum { A4 = 0x30,B4,C4,D4,E4,F4,G4,H4 } COORDINATE_4;
typedef enum { A5 = 0x40,B5,C5,D5,E5,F5,G5,H5 } COORDINATE_5;
typedef enum { A6 = 0x50,B6,C6,D6,E6,F6,G6,H6 } COORDINATE_6;
typedef enum { A7 = 0x60,B7,C7,D7,E7,F7,G7,H7 } COORDINATE_7;
typedef enum { A8 = 0x70,B8,C8,D8,E8,F8,G8,H8 } COORDINATE_8;

//  tipo int        0,    1,         2,      4
typedef enum { BIANCO, NERO, NO_COLORE, GRIGIO } COLORE;

//  tipo int          0,      1,       2,   3,       4,     5,     6
typedef enum { NO_PEZZO, PEDONE, CAVALLO,  RE, ALFIERE, TORRE, DONNA } TIPO_PEZZI;


// COLONNA TIPO INT 0-7
#define COL(X)                      ( X & 0xF )

// TRAVERSA TIPO INT 0-7                  
#define TRA(X)                      ( X >> 4 )

// CASA 0X88 DA COLONNA INT 0-7 E TRAVERSA INT 0-7                  
#define CASA_DA_COL_TRA(C,T)        ((16 * T) + C)

// CASA 0X88 DA COLONNA CHAR a-h E TRAVERSA CHAR 1-8                 
#define CASA_DA_MOSSA(C,T)          ((16 * (T-'1')) + (C-'a'))

// PER VEDERE SE UN PEZZO DI UNA CASELLA E' UNA TORRE, ALFIERE O DONNA
#define IS_SLIDE(X)                 ( X & 4 )

// per vedere se una casa e' nella scacchiera
#define IS_BOARD(X)                 !( X & 0x88 )

// side X  sotto scacco ?
#define SOTTO_SCACCO(X)             sotto_attacco( X^1, Pos.re[X] )

// l'assert normale da' errore di sistema
#define ASSERT(X)                   if(!(X)) my_assert( __LINE__ , __FILE__ )

#define MIN(A,B)                    ((A)<(B) ? (A):(B))


typedef struct{    
    unsigned char pezzo;
    unsigned char colore; 
    unsigned char flag;   
    unsigned char pezzo_loc;
    } casella;
	

typedef struct{
   unsigned char ep;                        // se non c'e' casa ep questa e' 0x8
   unsigned char cattura;                   // 3 bit per il pezzo e un bit per il colore della cattura
   unsigned char arrocco;
   unsigned char fifty;
   unsigned char pezzi_tot[2];
   unsigned char re[2];
   unsigned int materiale[2];
   unsigned long long int index;
  } posizione;
   
   
typedef struct{
   unsigned char from;
   unsigned char to;
   unsigned short score;
   unsigned short flag;
   } mossa;
   
typedef struct {    
   U64 index;
   short score;
   unsigned char flag;
   unsigned char depth;
   unsigned char from;
   unsigned char to;
   } tipo_tt;
   
   
struct {
   U64 pezzi[2][7][120];
   U64 ep[120];
   U64 arrocco[16];
   U64 side;
   } zobrist;
   
   
struct{
    int test;
	int m_residue;
	int time_control;
	double tempo_residuo;
	double tempo_mossa;
	} level;


#endif
